#!/bin/sh
#
# /etc/rc.d/rc.S:  System initialization script.
#

### Early userspace

PATH=/sbin:/usr/sbin:/bin:/usr/bin

# Mount proc and sysfs if needed.
mount -at proc,sysfs

### Get the family id to use below for selecting behaviour appropriate for that family.
FAMILYID="$(cut -f 2 /proc/navico_platform/family_id)"

# Squelch output during production.
if [ ! -f /dev/shm/debug ]; then
   exec 1> /dev/null
   exec 2> /dev/null
fi

case "${FAMILYID}" in
    Kiwi)
        # Could be KEY_P (025), KEY_ENTER (028), KEY_UP (103), KEY_DOWN (108)
        FACTDEFKEYS="103 108"
        NETWORK=false
        ;;
    Hobart)
        FACTDEFKEYS="025"
        NETWORK=false
        ;;
    Sylvester)
        FACTDEFKEYS="025"
        NETWORK=true
        ;;

    *)  FACTDEFKEYS="FIXME"
        NETWORK=true
        ;;
esac

# Init udev
# Temporary hack for handling Stingray2's ramdisk, which
# populates serial ports in a manner udev doesn't like.
rm -f /dev/ttyS*
if [ -x /etc/rc.d/rc.udev ]; then
  /etc/rc.d/rc.udev start
fi


# Disable complex scheduling for internal solid state storage.
# (leave the CFQ default alone for connectable USB hard disks)
if [ -d /sys/block ]; then
   for BLOCKDEV in \
      $(ls -d /sys/block/mmcblk* /sys/block/mtdblock* /sys/block/ubiblk* /sys/block/loop* 2>/dev/null) ;
   do
      echo noop > ${BLOCKDEV}/queue/scheduler
   done
fi

# Set system time
if [ -x /etc/rc.d/rc.time ]; then
   /etc/rc.d/rc.time start
fi

### Filesystem maintenance.

# Test to see if the root partition is read-only, like it ought to be.
READWRITE=no
if touch /fsrwtestfile 2>/dev/null; then
  rm -f /fsrwtestfile
  READWRITE=yes
else
  # echo "Testing root filesystem status:  read-only filesystem"

  echo "Remounting root device with read-write enabled."
  mount -w -v -n -o remount /
  if [ $? -gt 0 ] ; then
    echo
    echo "Attempt to remount root device as read-write failed!  This is going to"
    echo "cause serious problems."
    echo
    echo "Please press ENTER to continue, then start looking for the problem. " 
    read junk; 
  fi
fi

# If the mtab here is not a link to /proc/mounts, then it is stale.
if [ "$(stat -c '%N' /etc/mtab)" != "'/proc/mounts'" ]; then
   /bin/rm -f /etc/mtab*
   ln -sf /proc/mounts /etc/mtab
fi


# Mount non-root file systems in fstab, but not NFS or SMB 
# because TCP/IP is not yet configured, and not proc or sysfs
# because those have already been mounted.  Also check that
# devpts is not already mounted before attempting to mount
# it.  With a 2.6.x or newer kernel udev mounts devpts.
echo "Mounting non-root local filesystems:"
if /bin/grep -wq devpts /proc/mounts ; then
  mount -a -v -t nonfs,nosmbfs,nocifs,noproc,nosysfs,nodevpts
else
  mount -a -v -t nonfs,nosmbfs,nocifs,noproc,nosysfs
fi


# Set the memory manager to panic if memory is exhausted, then reboot after 3 seconds
echo 1 > /proc/sys/vm/panic_on_oom
echo 3 > /proc/sys/kernel/panic


# Populate some stuff in /tmp
mkdir -p /tmp/var/run/dbus
mkdir -p /tmp/var/cache/hald
mkdir /tmp/var/lock
mkdir /tmp/var/tmp
mkdir /tmp/oprofile
chmod 0777 /tmp
chmod 0777 /tmp/var/run

# Launch any 'runonce' commands.
for RUNONCE in $(ls /etc/rc.d/runonce.d/* 2>/dev/null) ; do
   if [ ! -f "${RUNONCE}" ]; then
      # Leave any weird stuff alone.
      continue
   fi
   if [ -x "${RUNONCE}" ]; then
      # Clean up after successful completion.
      "${RUNONCE}" && rm -f "${RUNONCE}"
      sync
   fi
done
unset RUNONCE

### Setup GPIO
if [ -x /etc/rc.d/rc.gpio ]; then
  /etc/rc.d/rc.gpio start
fi

### set lcd contrast
if [ -r /etc/NOS/lcd_contrast ]; then
  cat /etc/NOS/lcd_contrast > /sys/class/leds/lcd-contrast/brightness
fi

### turn on GPS
if [ ${FAMILYID} = "Sylvester" ]; then
  if [ -x /etc/rc.d/rc.igps ]; then
    # detection can take up to 5 secs, run in background
    /etc/rc.d/rc.igps start >/var/run/rc.igps.log 2>&1 &
  fi

  ### turn on and (optionally) reprogram Sonar
  if [ -x /etc/rc.d/rc.stm32 ]; then
    /etc/rc.d/rc.stm32 start >/var/run/rc.stm32.log 2>&1 &
  fi
fi

if grep -i -q CreateCrashLog=true /etc/NOS/Config.ini; then
  touch /dev/shm/enablecrashlog
fi

# If user wants to load defaults OR if home partition has not been generated yet, 
# then delete the contents of the ubi?:home partition
if [ "$(cat /dev/shm/bootkeys)" = "${FACTDEFKEYS}" ] || [ ! -e /home/.fscreated ]; then
   echo "User has requested factory defaults. Deleting home partition..."

   # Work out which UBI volume home is mounted on
   UBIDEV="$(grep /home /proc/mounts | cut -f 1 -d ' ' | cut -f 1 -d ':')"
   UBIDEV="${UBIDEV:-ubi0}"
   for syspath in /sys/class/ubi/${UBIDEV}_?; do
     if [ "$(cat ${syspath}/name)" = "home" ]; then
       vol_num=$(echo ${syspath} | cut -d "_" -f 2)
       VOLDEV="/dev/${UBIDEV}_${vol_num}"
       unset vol_num
       break
     fi
   done
   unset syspath
   unset UBIDEV

   if [ $VOLDEV ]; then
      # Unmount current home fs
      umount /home
      # Truncate ubi volume
      ubiupdatevol ${VOLDEV} -t
      # Create new blank fs, with no space reserved for root
      mkdir home_tmp
      mkfs.ubifs --reserved=0 -r home_tmp ${VOLDEV}
      rm -r home_tmp
      # Mount new fs
      mount /home

      # Add flag so we remember fs is generated.
      # Need this because FS is auto-generated at mount time if it is blank, the FS defaults to reserving blocks for
      # root (which we don't want), and there is no user space tune fs tool (though it seems that one is possible
      # using some ioctl's, but it has never been implimented)
      touch /home/.fscreated
      sync
   fi
   unset VOLDEV
fi
unset FACTDEFKEYS

if [ -x /usr/sbin/updatesplashscreen ]; then
  /usr/sbin/updatesplashscreen
fi


### Log system messages when in debug mode
if [ -e /dev/shm/enablecrashlog ] || [ -e /dev/shm/debug ]; then
  [ -x /etc/rc.d/rc.syslog ] && /etc/rc.d/rc.syslog start
  [ -x /etc/rc.d/rc.debuglog ] && /etc/rc.d/rc.debuglog start
fi


# Set the system hostname
if $NETWORK; then
   if [ -s /etc/HOSTNAME ]; then
      /bin/hostname -- $(cut -f 1 -d . /etc/HOSTNAME)
   else
      # Assemble a default
      MACADDR="$(cut -f 4,5,6 -d ':' /etc/NOS/MAC 2>/dev/null | tr ':' '-' 2>/dev/null)"
      echo ${FAMILYID}${MACADDR:+-${MACADDR}}.invalid.domain > /etc/HOSTNAME
      fsync /etc/HOSTNAME
      /bin/hostname -- "$(cut -f 1 -d . /etc/HOSTNAME)"
   fi

   # Also update the hosts file if necessary
   if [ -s /etc/HOSTNAME ] && ! grep -q "$(head -n 1 /etc/HOSTNAME)" /etc/hosts; then
      HOSTNAME="$(head -n 1 /etc/HOSTNAME)"
      HOSTDOMAIN="$(echo "${HOSTNAME}" | cut -f 2- -d '.')"

      # Strip out all the old localhosts
      sed '/^[ ]*127.0.0.1/d' /etc/hosts > /etc/hosts.new

      # Just stick the new ones on the end.
      echo "127.0.0.1         localhost" >> /etc/hosts.new
      echo "127.0.0.1         ${HOSTNAME} ${HOSTNAME}${HOSTDOMAIN:+.${HOSTDOMAIN}}" >> /etc/hosts.new
      fsync /etc/hosts.new
      mv /etc/hosts.new /etc/hosts
      unset HOSTNAME HOSTDOMAIN
   fi
fi

### Set default key repeat rates (delay and period in ms) for Stingray2
if [ ${FAMILYID} == "Kiwi" ]; then
    for kbd in /dev/input/keypad; do
      if [ -e ${kbd} ]; then
        inpdev --device=$kbd --rptdly 500 --rptprd 125
      else
	echo "Input device ${kbd} doesn't exist";
      fi
    done
fi

### start D-bus
if [ -x /etc/rc.d/rc.messagebus ]; then
  /etc/rc.d/rc.messagebus start  
fi

### bring up network interfaces
if $NETWORK && [ -x /etc/rc.d/rc.net ] || [ ${FAMILYID} == "Kiwi" ]; then
  /etc/rc.d/rc.net start  

   # If telnet login is required, try to dump the IP address for eth0 out the serial port
   if [ -e /dev/shm/debug ]; then
      unset IPADDR
      (  while [ -z "${IPADDR}" ];
         do
            IPADDR=$(
               ifconfig | \
               grep -A2 eth0 | \
               grep "inet addr:" | \
               sed 's/.*inet addr://; s/ .*//'
            )
            sleep 1
         done
         if [ -n "${IPADDR}" ]; then
            SERIALSPEED=$(stty -F /dev/tts/0 speed)
            stty -F /dev/tts/0 115200
            echo "My IP address for eth0 is ${IPADDR}" > /dev/tts/0
            stty -F /dev/tts/0 "${SERIALSPEED}"
         fi
      ) &
   fi
fi

if $NETWORK && [ -e /dev/shm/debug ]; then
  ### run telnetd
  /usr/sbin/telnetd

  ### run sshd
  [ -x /etc/rc.d/rc.sshd ] && /etc/rc.d/rc.sshd start
fi

# Start NFS-related services.
if $NETWORK && [ -x /etc/rc.d/rc.rpc ]; then
   /etc/rc.d/rc.rpc start
fi
if $NETWORK && [ -x /etc/rc.d/rc.nfsd ]; then
   /etc/rc.d/rc.nfsd start
fi

### start hal
if [ -x /etc/rc.d/rc.hald ]; then
  /etc/rc.d/rc.hald start
fi

# If we've reenabled the serial console,
# move the ttyS0 symlink so NOSApp can't find it (and init will).
if [ -f /etc/nmea0183debug ] && [ ${FAMILYID} = "Sylvester" ]; then
   mv /dev/ttyS0 /dev/ttymxc0
   ln -sf /dev/null /dev/ttyS0
else
   # Have init open a console prompt on the default VC.
   # (probably the dummy virtual console.)
   ln -sf /dev/null /dev/ttymxc0
fi

# Enable core-dump handling
[ -x /etc/rc.d/rc.navico-coredump ] && /etc/rc.d/rc.navico-coredump start
# Enable log to media
[ -x /etc/rc.d/rc.navico-logtomedia ] && /etc/rc.d/rc.navico-logtomedia start

if [ -x /sbin/watchdog ]; then
  /sbin/watchdog -t 2 -T 15 /dev/watchdog
fi

### Run the application initialization script
# Background waiting, so we don't prevent login prompt
(
  # Wait for STM32 micro updates to be complete (stingray2, condor & cougar units)
  if pgrep -f rc.stm32 > /dev/null; then
    echo -n "Waiting for stm32 software upgrade..."
    while pgrep -f rc.stm32 > /dev/null; do
      usleep 25000
    done
    echo Done
  fi

  # Wait for the GPS upgrades to be complete
  if pgrep -f rc.igps > /dev/null; then
    echo -n "Waiting for igps software upgrade..."
    while pgrep -f rc.igps > /dev/null; do
      usleep 25000
    done
    echo Done
  fi

  if [ -x /etc/rc.d/rc.app ]; then
    /etc/rc.d/rc.app start
  fi
) &
