# A set of common helper functions usefull for iMX28 updaters

### Immediate execution

UPDDIR="$(dirname "${UPDFILE}")"
THISCODENAME="$(grep CODENAME /etc/updater/updater.conf | cut -d= -f2)"
FBSPLASH_SIZE=$(cut -f 2 /proc/navico_platform/disp_size | cut -c0)
 
### Functions


nand_geometry() {
  local ERASESIZE=$(($(cat /sys/class/mtd/mtd0/erasesize) / 1024))
  local PAGESIZE=$(($(cat /sys/class/mtd/mtd0/writesize) / 1024))

  echo ${PAGESIZE}k${ERASESIZE}k
}

nand_page_size() {
  cat /sys/class/mtd/mtd0/writesize
}

nand_erase_size() {
  cat /sys/class/mtd/mtd0/erasesize
}

## Try and figure out the NAND size that is present
nand_size() {
  local SIZES=$(cat /proc/mtd | grep mtd | cut -d " " -f 2)
  local NANDSIZE=0

  # Sum the MTD partitions. This is a bit flawed, because there is no rule that mtd partitions
  # must use the whole device, or that the mtd partitions cannot overlap. Our mx51 platform
  # uses the whole device in non-overlapping partitions, so this will work.
  for size in $SIZES; do
    size=$((0x$size))
    NANDSIZE=$(($NANDSIZE + $size))
  done
  unset size

  echo $NANDSIZE
}

# Finds the ubi in format of eg ubi0_1
find_ubi_volume_num_from_name() {
  volume_name=$1

  # Scan the names of the ubi volumes, to check if this component is a volume image
  # Relies on UBI volumes not getting the same name as mtd partitions
  local UBIVOL=
  for syspath in /sys/class/ubi/ubi?_?; do
    if [ "$(cat ${syspath}/name)" = "${volume_name}" ]; then
      UBIVOL=$(echo ${syspath} | cut -d "/" -f 5)
      break
    fi
  done
  unset syspath
  echo $UBIVOL
}

# Expects bootloader file name in $1, minimum bootloader version in $2
update_boot() {
  if [ "$2" = "-1" ] || [ "$(cat /proc/navico_platform/boot_ver)" -lt "$2" ]; then
    local UBOOTMTD="$(grep -i '"uboot"' /proc/mtd | sed 's/^mtd//; s/:.*//')"

    mkdir boottmp
    tar xOf "${UPDFILE}" $1 | xzcat | tar x -C boottmp

    local ENVPAD=boottmp/envpad-arm
    local ENV=boottmp/all.env
    local UBOOT_BIN=boottmp/boot.bin

    # Attempt to preserve the mtd layout that is currently on the device
    local MTDPARTS_NUM=
    local CURRENT_MTDPARTS=$(sed 's/\(.*\)\(mtdparts=[(),:0-9A-Za-z-]*\)\(.*\)/\2/; s/mtdparts=//' /proc/cmdline)
    if [ "$THISCODENAME" = "sylvester" ]; then
      MTDPARTS_NUM=1
    else
      MTDPARTS_NUM=0
    fi
    sed -i 's/mtdparts_'${MTDPARTS_NUM}'=.*/mtdparts_'${MTDPARTS_NUM}="$CURRENT_MTDPARTS"'/' $ENV

    local ENV_OFFSET=$( ${ENVPAD} --info | sed 's/.*offset=\([0-9]*\).*/\1/' )
    ${ENVPAD} --info

    #bootloader at 1MB offset in FLASH, so subtract to get relative address
    local JBOFFSET=$(($ENV_OFFSET - (1024*1024))) 

    (
      dd if=${UBOOT_BIN} bs=${JBOFFSET} conv=notrunc conv=sync count=1 
      cat ${ENV} | ${ENVPAD} -
    ) > uboot.bin

    flash_eraseall /dev/mtd$UBOOTMTD
    cat uboot.bin | chroot /mnt nandwrite --pad /dev/mtd$UBOOTMTD -

    rm uboot.bin
    rm -fR boottmp/
  fi
}

# expects mtd name in $1, file name in $2
update_mtd_image() {
  local BMPMTD="$(grep -i \"$1\" /proc/mtd | sed 's/^mtd//; s/:.*//')"
  if [ -n "${BMPMTD}" ]; then
    flash_eraseall /dev/mtd"${BMPMTD}"
    tar -xOf "${UPDFILE}" $2 | xzcat | chroot /mnt nandwrite --pad /dev/mtd$BMPMTD -
  fi
}

start_splash_screen() {
  sha1check fbsplash.tar.xz && tar -xOf "${UPDFILE}" fbsplash.tar.xz | xzcat | tar -x || true
  sha1check fbsplash-${THISCODENAME}.tar.xz && tar -xOf "${UPDFILE}" fbsplash-${THISCODENAME}.tar.xz | xzcat | tar -x || true
  if [ -e updating${FBSPLASH_SIZE}.ppm ]; then
    mkfifo fbsplash.fifo
    tar -xOf "${UPDFILE}" fbsplash.tar.xz | xzcat | tar -x || \
       tar -xOf "${UPDFILE}" fbsplash-${THISCODENAME}.tar.xz | xzcat | tar -x || \
       true
    fbsplash -s updating${FBSPLASH_SIZE}.ppm -i fbsplash${FBSPLASH_SIZE}.cfg -f fbsplash.fifo &
    sleep 2
  fi
}

display_progress () {
   if [ -e fbsplash.fifo ]; then
     echo $1 >> fbsplash.fifo
   fi
   return 0;
}

display_done_image() {
   if [ -e fbsplash.fifo ]; then
     echo exit >> fbsplash.fifo
     sleep 2
     rm fbsplash.fifo
   fi
   fbsplash -s updatefinished${FBSPLASH_SIZE}.ppm || true
   return 0;
}

display_error_image() {
   if [ -e fbsplash.fifo ]; then
     echo exit >> fbsplash.fifo
     sleep 2
     rm fbsplash.fifo
   fi
   fbsplash -s updateerror${FBSPLASH_SIZE}.ppm || true
   return 0;
}

wait_for_media_reboot() {

  # Get the device number (in decimal) of updater
  MEDIA_DEV=$(stat -c '%d' "${UPDFILE}")
  # Generate a regex to use with /proc/partitions
  MEDIA_RE="^[[:space:]]*$(($MEDIA_DEV/256))[[:space:]]\+"	# major
  MEDIA_RE="${MEDIA_RE}$(($MEDIA_DEV%256))[[:space:]]\+"	# minor
  MEDIA_RE="${MEDIA_RE}[[:digit:]]\+[[:space:]]\+mmcblk"	# blocks, name

  set +x  # squelch annoying infinite loop.
  # Omit -q(uiet) flag for first grep
  if grep "${MEDIA_RE}" /proc/partitions; then
     while grep -q "${MEDIA_RE}" /proc/partitions; do
       :
     done
  fi

  echo 'reboot -f' > ~/.updater/bootaction
}

