#!/bin/sh
# Update script

# System handshake magic
UNAME=armv5tejl
CODENAME=kiwi
DATESTAMP=20150924084647
MINBOOTVER=0

# This progress bar delay is just used as an approximate speed for moving the progress bar
# After the bar has passed 75%, it will slow to 1/2 this rate, and 1/4 this rate after 90%
PROGRESSBARDELAY=0.75

# Print out the updater version. This needed by the factory to validate the correct version is being
# programmed when doing automated CPU card programming
# NOTE: makeupd.sh will modify this line
echo UPDATEVERSION=VesselView4-5.0-43.579-32766-r1

# TODO: Detect need for UBI repair automatically. This should only ever need to be done once 
#       at the factory. Probably use logic of if (!ubiattach) || (volumecount != 3) { ubirepair }


DEBUG=${DEBUG:-false}
if [ -e /dev/shm/debug ]; then
   DEBUG=true
fi
${DEBUG} && set -x

# If anything goes awry, exit early.
set -e

# We expect to be invoked from the /tmp directory.
# We expect to have working /proc and /sys directories, as well as a populated /dev

### This upgrade script expects the following environment variables to be present:
# UPDFILE -- a path to the '.upd' update file.
#    ex:  UPDFILE = /path/to/foo.upd
#    This file path is found during the discovery phase of the initramfs
#    and will be relative to the ramfs root.
if [ -z "${UPDFILE}" ]; then
   echo "Internal error: Bad environment.  UPDFILE was not set."
   exit 1;
fi



### This file also encapsulates SHA1 sums of the componets it expects to operate upon.
# These need to be checked before the script proceeds.
# TODO: Use hardware-accelerated SHA1
# TODO: Test the system sha1sum for tampering.
SHA1CMD=sha1sum
sha1check () {
   if [ -z "${1}" ]; then
      echo "no file to check, aborting"
      return 1;
   fi

   local EXTERNAL=0
   if [ "${1}" = "-external" ]; then
      EXTERNAL=1
      shift 1
   fi

   THISSHA1="$(
      grep -A 500 "^###SHA1MAGIC###$" "${0}" | \
         grep -o "[^ ]\{40\}  ${1}$"
   )"
   if [ "$?" -ne "0" ]; then
      echo "File \"${1}\" not found in internal hash table."
      return 1
   fi
   THISSHA1="$(echo "${THISSHA1}" | cut -f 1 -d ' ' )"
   ${DEBUG} && echo "Stored SHA1 for ${1} is ${THISSHA1}"

   if [ "${EXTERNAL}" = 1 ]; then
      if [ ! -r "$(dirname "${UPDFILE}")/${1}" ]; then
         echo "Couldn't open external file ${1}"
         return 1
      fi
      LOCALSHA1="$("${SHA1CMD}" "$(dirname "${UPDFILE}")/${1}" | cut -f 1 -d ' ')"
   else
      LOCALSHA1="$(tar -xOf ${UPDFILE} ${1} | "${SHA1CMD}" | cut -f 1 -d ' ' )"
   fi

   ${DEBUG} && echo "Calculated SHA1 for ${1} is ${LOCALSHA1}"

   if [ "${THISSHA1}" != "${LOCALSHA1}" ]; then
      echo "ERROR: Update corrupt.  File \"${1}\" failed checksum."
      echo "Please delete this update and replace it with a new copy."
      return 1;
   else
      ${DEBUG} && echo "DEBUG: successful sha1sum check: ${1}"
   fi
   return 0;
}


sha1check libupdate.sh || {
  echo "fail"
}

tar -xf "${UPDFILE}" libupdate.sh

. ./libupdate.sh

start_splash_screen

# Background progress loop.
PROGRESS=0
(  set +x
   while [ ! -f /stop-progress-flag.$$ ] && \
      [ ${PROGRESS} -le 98 ];
   do
      PROGRESS=$((${PROGRESS}+1))
      display_progress ${PROGRESS}
      sleep ${PROGRESSBARDELAY}
      if [ ${PROGRESS} -ge 75 ]; then
         # 2x delay
         sleep ${PROGRESSBARDELAY}
      fi
      if [ ${PROGRESS} -ge 90 ]; then
         # 4x delay
         sleep ${PROGRESSBARDELAY}
         sleep ${PROGRESSBARDELAY}
      fi
   done
) &


###  Determine unit type

# What payloads are we carrying?
# Search our signature table.
COMPONENTLIST="$( \
   grep -A 42 -m 1 '^###SHA1MAGIC###$' "${0}" | \
   sed 's/[^ ]\{40\}  //; /###SHA1MAGIC###/d' | \
   sort | \
   uniq
)"

update_corrupt() {
    echo "ERROR:  This update is damaged.  Halted!"
    display_error_image
    while true; do
       sleep 42d
    done
}

# Check all the components are valid before doing anything
for COMPONENT in $COMPONENTLIST; do
  sha1check $COMPONENT || update_corrupt
done

cat /proc/mtd

# Do the root file-system first, so we get the tools on there for later use
if [ ! -z "$(echo ${COMPONENTLIST} | grep root.tar.xz)" ]; then
  # Delete and re-generate default file-system
  ubiupdatevol -t /dev/$(find_ubi_volume_num_from_name root)
  mount /mnt

  tar -xOf "${UPDFILE}" root.tar.xz | xzcat | tar -x -C /mnt

  umount /mnt
  sync
fi

mount /mnt
mount --bind /dev /mnt/dev
mount --bind /sys /mnt/sys

chroot /mnt sh -l -c "ldconfig"

if [ ! -z "$(echo ${COMPONENTLIST} | grep kernel.bin)" ]; then
  update_mtd_image kernel kernel.bin
fi

if [ ! -z "$(echo ${COMPONENTLIST} | grep boot.bin)" ]; then
  #update_boot $MINBOOTVER
  update_boot boot.bin -1
fi

FBSPLASH_SIZE=$(cut -f 2 /proc/navico_platform/disp_size | cut -c0)

if [ ! -z "$(echo ${COMPONENTLIST} | grep logo${FBSPLASH_SIZE}.bin)" ]; then
  update_mtd_image logo logo${FBSPLASH_SIZE}.bin
fi


umount /mnt/sys
umount /mnt/dev
umount /mnt


echo "Update successful"

touch /stop-progress-flag.$$
sleep 1
display_progress 100
sleep 1

display_done_image

wait_for_media_reboot

# Done!
exit 0
exit 0
###SHA1MAGIC###
f79228bac78330e0e3adf5602152e8ffb246577a  libupdate.sh
befa147ce218b463e3265402a0456a25f95e54a9  boot.bin
7cdc24797b82f13b7ff63f062be2101d04faff26  kernel.bin
2cc4035e14a93abbfbaa27bb628bda6b39d1f1be  root.tar.xz
47cdc54d50d1488fb50148e2dfe9453c3162dd53  fbsplash.tar.xz
